@@ -561,6 +561,22 @@ class PickelingTest(TestCase):
         self.assertIsInstance(obj, thread.LockType)
         self.assertTrue(obj.locked())
         
+    def testThreadLockAcquire(self):
+        self.builtinMethodTest(thread.allocate_lock(), "acquire")
+    def testThreadLockRelease(self):
+        self.builtinMethodTest(thread.allocate_lock(), "release")
+    def testThreadLockLocked(self):
+        self.builtinMethodTest(thread.allocate_lock(), "locked")
+
+    def builtinMethodTest(self, obj, methodName):
+        method = getattr(obj, methodName)
+        p = self.dumpWithPreobjects(method, obj, dis=False)
+        restoredMethod, restoredObj = self.pickler.loads(p)
+        self.assertIsInstance(restoredObj, thread.LockType)
+        self.assertIsNot(restoredObj, obj)
+        self.assertIs(restoredMethod.__self__, restoredObj)
+        self.assertEqual(restoredMethod, getattr(restoredObj, methodName))
+        
     def testObject__delattr__(self):
         orig = object.__delattr__
         p = self.dumpWithPreobjects(None,orig, dis=False)
@@ -758,6 +774,27 @@ class PickelingTest(TestCase):
         self.assertIsNot(obj, orig)
         self.assertTupleEqual((1, 2, 3), obj(target))
 
+    def testXrange(self):
+        orig = xrange(4,10,2)
+        p = self.dumpWithPreobjects(None,orig, dis=False)
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsInstance(obj, xrange)
+        self.assertListEqual(list(obj), list(orig))
+
+    def testRangeIterator(self):
+        x = xrange(2,10,2)
+        orig = iter(x)
+        lorig = []
+        lorig.append(orig.next())
+        lorig.append(orig.next())
+        lobj = lorig[:]
+        p = self.dumpWithPreobjects(None,orig, dis=False)
+        lorig.extend(orig)
+        self.assertListEqual(lorig, list(x))
+        obj = self.pickler.loads(p)[-1]
+        self.assertIsInstance(obj, type(orig))
+        lobj.extend(obj)
+        self.assertListEqual(lobj, lorig)
         
     # Tests for pickling classes
     
@@ -798,6 +835,11 @@ class PickelingTest(TestCase):
         self.assertEqual(nt.a, 1)
         self.assertEqual(nt.b, 2)
         
+    def testExceptionClass(self):
+        class TestException(BaseException):
+            attribute = "attribute value"
+        self.classCopyTest(TestException, dis=False)
+
     def classCopyTest(self, origCls, dis=False):
         p = self.dumpWithPreobjects(None,origCls, dis=dis)
         cls = self.pickler.loads(p)[-1]
@@ -894,6 +936,13 @@ class PickelingTest(TestCase):
         self.assertIsNot(restored, orig)
         self.assertIsInstance(restored, orig.__class__)
         
+class SPickleToolsTest(TestCase):
+    def testModule_for_globals(self):
+        pt = _sPickle.SPickleTools()
+        self.assertIs(pt.module_for_globals({}), None)
+        self.assertIs(pt.module_for_globals(_sPickle.__dict__), _sPickle)
+        self.assertIs(pt.module_for_globals(pt.module_for_globals), _sPickle)
+        self.assertIs(pt.module_for_globals(pt.module_for_globals, withDefiningModules=True), _sPickle)
 
 if __name__ == "__main__":
     import unittest
